unit minedoutspielfeld;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, minedoutbase;

type
 { TSpielfeld }

  TSpielfeld = class
    private
      FSpielfeld : array [WESTEN..OSTEN,
                                          SUEDEN..NORDEN]
                                          of TFeldinhalt;
      FStartposition : TPos;
      function getStartposition : TPos;

    public
      constructor Create;
      destructor Destroy; override;
      procedure LegeMinen(AnzahlMinen : Integer);
      function IstBewegungZulaessig(Punkt : TPos;
        Richtung : TRichtung) : Boolean;
      property Startposition : TPos read getStartposition;
      function MinenInDerNaehe(Punkt : TPos) : Integer;
      function getInhalt(Punkt : TPos) : TFeldinhalt;
      { TODO : Diese Procedure löschen, wenn sie nicht mehr gebraucht wird }
      procedure ZeichneSpielfeld; // Nur für Testzwecke!!!
  end;



implementation

{ TSpielfeld }

function TSpielfeld.getStartposition: TPos;
begin
  Result := FStartposition;
end;

constructor TSpielfeld.Create;
var
  i, j : Integer;
begin
  (* FSpielfeld als Leer initialisieren *)
  for i:= WESTEN to OSTEN do
    for j:= SUEDEN to NORDEN do
      FSpielfeld[i,j] := TFeldinhalt.Leer;

  (* Starposition FSpieler festlegen *)
  FStartposition.X:= OSTEN Div 2;
  FStartposition.Y:= SUEDEN;
  FSpielfeld[FStartposition.X, FStartposition.Y] := TFeldinhalt.Spieler;

  (* FZiel festlegen *)
  FSpielfeld[OSTEN Div 2, NORDEN] := TFeldinhalt.Ziel;

  end;

destructor TSpielfeld.Destroy;
begin
  inherited Destroy;
end;


procedure TSpielfeld.LegeMinen(AnzahlMinen : Integer);
var
  y: Integer;
  x: Integer;
begin
  (* Minen legen *)
    Randomize;
    while AnzahlMinen > 0 do
      begin
        x := Random(OSTEN);
        y := Random(NORDEN);
        (* Minen dürfen nicht auf der ersten und letzten Linie liegen! *)
        if (y > SUEDEN) and (y < NORDEN) then
          if FSpielfeld[x, y] = Leer then
            begin
              FSpielfeld[x, y] := Mine;
              Dec(AnzahlMinen);
            end;
        end;
end;

function TSpielfeld.IstBewegungZulaessig(Punkt: TPos;
  Richtung: TRichtung): Boolean;
begin
  Result := false;
  Case Richtung of
    TRichtung.rNorden:
      Result := Punkt.Y < NORDEN;
    TRichtung.rSueden:
      Result := Punkt.Y > SUEDEN;
    TRichtung.rOsten:
      Result := Punkt.X < OSTEN;
    TRichtung.rWesten:
      Result := Punkt.X > WESTEN;
  end;
end;

function TSpielfeld.MinenInDerNaehe(Punkt : TPos) : Integer;
begin
  Result := 0;
  // Norden
  if Punkt.Y < NORDEN then
    if FSpielfeld[Punkt.X, Punkt.Y+1] = TFeldinhalt.Mine then
      Inc(Result);
  // Süden
  if Punkt.Y > SUEDEN then
    if FSpielfeld[Punkt.X, Punkt.Y-1] = TFeldinhalt.Mine then
      Inc(Result);
  // Osten
  if Punkt.X < OSTEN then
    if FSpielfeld[Punkt.X+1, Punkt.Y] = TFeldinhalt.Mine then
      Inc(Result);
  // Westen
  if Punkt.X > WESTEN then
    if FSpielfeld[Punkt.X-1, Punkt.Y] = TFeldinhalt.Mine then
      Inc(Result);

end;

function TSpielfeld.getInhalt(Punkt: TPos): TFeldinhalt;
begin
  Result := FSpielfeld[Punkt.X, Punkt.Y];
end;

procedure TSpielfeld.ZeichneSpielfeld;
var
  i, j : Integer;
  Feld : String;
begin
  for i := NORDEN DownTo SUEDEN do
    begin
      for j := WESTEN to OSTEN do
      begin
        Case FSpielfeld[j,i] of
          TFeldinhalt.Ziel: Feld := 'Z';
          TFeldinhalt.Spieler: Feld := 'S';
          TFeldinhalt.Mine: Feld := 'X';
          TFeldinhalt.Leer: Feld := '.';
        else
          Feld := '?';
        end;
        Write(Feld);
      end;
      Writeln;
    end;
  Writeln('press Enter...');
  // Readln;
end;





end.

